%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% combustion.m - setting parameters and sets related to combustion
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions and scripts used:
% GDXMRW:       writegdx.m      writing data to gdx file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Heat capacities (based on enthalpy equations)
% From HYSYS H = a+bT+cT^2+dT^3+eT^4+fT^5 [kJ/kg], T [K]
% Valid for T between -270 and 5000 C. [b c d e f]
HCp_coeffs.H2O=[1.9145 -3.9574e-4 8.76206e-7 -4.95055e-10 1.03846e-13];
HCp_coeffs.CO=[1.0739 -1.7265e-4 3.02226e-7 -1.37526e-10 2.00356e-14];
HCp_coeffs.CO2=[0.618139 4.84485e-4 -1.49353e-7 2.2905e-11 -1.37045e-15];
HCp_coeffs.H2=[13.8376 2.999806e-4 3.458931e-7 -9.712927e-11 7.731201e-15];
HCp_coeffs.CH4=[2.36459 -2.13247e-3 5.6618e-6 -3.72476e-9 8.60896e-13];
HCp_coeffs.N2=[.982747 9.71424e-5 -4.15795e-10 -3.65548e-12 4.05013e-16];
HCp_coeffs.O2=[.952 -2.81129e-4 6.55206e-7 -4.52296e-10 1.08756e-13];
for j = components
    i=char(j);
    eval(['HCp_coeffs.' i '=HCp_coeffs.' i '.*[1 2 3 4 5];']);
end

TIT=1350+C2K;
if lowerTIT>0
    TIT=lowerTIT+C2K;
end
T_air=415+C2K;
T_fuel=200+C2K;

% Calculating the amount of heat needed to heat one kg of air from the
% temperature out of the compressor to TIT
a=HCp_coeffs.O2;
q_comb_air=airO2mass*(a(1)*(TIT-T_air)+.5*a(2)*(TIT^2-T_air^2)+...
    a(3)/3*(TIT^3-T_air^3)+.25*a(4)*(TIT^4-T_air^4)+...
    .2*a(5)*(TIT^5-T_air^5));                               % [kJ/kg]
a=HCp_coeffs.N2;
q_comb_air=q_comb_air+airN2mass*(a(1)*(TIT-T_air)+...
    .5*a(2)*(TIT^2-T_air^2)+a(3)/3*(TIT^3-T_air^3)+...
    .25*a(4)*(TIT^4-T_air^4)+.2*a(5)*(TIT^5-T_air^5)); % [kJ/kg]

% Calculating the amount of heat needed to heat one kg from the preheat
% temperature to TIT for each combustion component
for j=components
    i=char(j);
    eval(['a=HCp_coeffs.' i ';']);
    q=(a(1)*(TIT-T_fuel)+.5*a(2)*(TIT^2-T_fuel^2)+...
        a(3)/3*(TIT^3-T_fuel^3)+.25*a(4)*(TIT^4-T_fuel^4)+...
        .2*a(5)*(TIT^5-T_fuel^5));
    eval(['CpdeltaT.' i '=q;']); % [kJ/kg/K]
end
CpDT=[CpdeltaT.H2O; CpdeltaT.CO2; CpdeltaT.O2; CpdeltaT.N2];   %[kJ/kg/K]

% q=[CpdeltaT.H2O                                 % H2O
%     CpdeltaT.CO2-.5*CpdeltaT.O2                 % CO
%     CpdeltaT.CO2                                % CO2
%     CpdeltaT.H2O-.5*CpdeltaT.O2                 % H2
%     CpdeltaT.CO2+2*(CpdeltaT.H2O-CpdeltaT.O2)   % CH4
%     CpdeltaT.N2];                               % N2

% Stoichiometry of the combustion reactions
combreaction=zeros(v,4);
combreaction(CCU_H2O,1)=1                                 *split.H2O(1);
combreaction(CCU_CO,:)=[0 1*M.CO2 -.5*M.O2 0]       /M.CO *split.CO (1);
combreaction(CCU_CO2,2)=1                                 *split.CO2(1);
combreaction(CCU_H2,:)=[1*M.H2O 0 -.5*M.O2 0]       /M.H2 *split.H2 (1);
combreaction(CCU_CH4,:)=[2*M.H2O 1*M.CO2 -2*M.O2 0] /M.CH4*split.CH4(1);
combreaction(CCU_N2,4)=1                                  *split.N2 (1);

% The composition of air, with respect to exhaust components
aircomp=[exh_O2 airO2mass;
         exh_N2 airN2mass];

% Lower heating value in the gas turbine fuel
LHV=zeros(v,1);
LHV(CCU_CO)=LHV_CO*split.CO(1);     %[kJ/kg]
LHV(CCU_H2)=LHV_H2*split.H2(1);     %[kJ/kg]
LHV(CCU_CH4)=LHV_CH4*split.CH4(1);  %[kJ/kg]

% Set of exponents for Cp (or H) equations
expset=(1:5)'+length(uels);
uels(expset)={'exp1', 'exp2', 'exp3', 'exp4', 'exp5'};

% Set of combustion related variables
v_comb=union(exhaust_set, [outlettemp_set; W_set; comb_airflow]);

% Writing data to file
combreaction_gdx=zeros(v*length(exhaust_set),3);
k=1;
for i=1:v
    for j=1:4
        combreaction_gdx(k,1)=i;
        combreaction_gdx(k,2)=exhaust_set(j);
        combreaction_gdx(k,3)=combreaction(i,j);
        k=k+1;
    end
end
writegdx('combustion.gdx', ...
    'set',       'v_comb',       v_comb, ...
    'set',       'exhcomp',      exhaust_set , ...
    'set',       'T_out',        outlettemp_set, ...
    'set',       'W',            W_set, ...
    'set',       'comb_airflow', comb_airflow, ...
    'parameter', 'combreaction', combreaction_gdx, ...
    'parameter', 'q_comb_air',   q_comb_air, ...
    'parameter', 'air',          aircomp, ...
    'parameter', 'LHV',          [(1:v)' LHV], ...
    'parameter', 'CpDT',         [exhaust_set CpDT], ...
    'parameter', 'tempscale',    tempscale, ...
    uels);