********************************************************************************
* ircc.gms - Integrated Reforming Combined Cycle
********************************************************************************
* OTHER FILES NEEDED:
* GAMS SCRIPTS:
*       combustion.gms, air.gms, turbo.gms, hrsgconnected.gms (using hr.gms)
* GDX DATA FILES:
*       masterset.gdx, regcoeffs.gdx, modelparameters.gdx,
*       (and indirectly: combustion.gdx, air.gdx, turbo.gdx, hrsgconnected.gdx)
********************************************************************************

*Reading sets and parameter values from gdx files
*Reading the full set...
$Gdxin masterset.gdx
Set      v_all                    indices for ALL variables;
$load v_all
Set      v_bounded(v_all)         the variables with bounds;
$load v_bounded
Parameters
         lower(v_all)             lower bounds
         upper(v_all)             upper bounds
         objcoeffs(v_all)         objective function coefficients
         objtype                  the type of objective (power or efficiency)
         maxtime                  maximum solution time;
$load lower, upper, objcoeffs, objtype, maxtime
$Gdxin
*Reading regression coefficients and related sets and parameters
$Gdxin regcoeffs.gdx
Set      v(v_all)                 indices for all variables in regression;
$load v
Sets
         iv(v)                    indices for the independent variables
         dv(v)                    indices for the dependent variables
         dv_all(v_all)
         b                        indices for all regression coefficients;
$load iv, dv, b
Parameters
         betas(b,dv)              regression coefficients
         high(v)                  highest value of variables in the data set
         low(v)                   lowest value of variables in the data set;
Sets
         lb(iv,b)                 indices for linear regression coefficients
         qb(iv,b)                 indices for quadratic regression coefficients
         bb(iv,iv,b)              indices for bilinear regression coefficients;
$load betas, lb, qb, bb, high, low
$Gdxin
*Reading objective coefficients ("costs"), upper and lower bounds, linear
*constraint coefficients and bounds and starting point, with related sets
$Gdxin modelparameters.gdx
Sets
         feedflow(v_all)         the flow rate of the methane feed
         lc                      indices for the linear constraints
         ec                      indices for linear equality constraints
         pr                      indices for product constraints;
$load lc, feedflow, ec, pr
alias(v_all, v_all2, v_all3);
Parameters
         linconstr(lc, v_all)    coefficient matrix for linear constraints
         linconstrbound(lc)      vector of bounds for the linear constraints
         eqconstr(ec, v_all)     coefficient matrix for linear equality constrs.
         eqconstrconst(ec)       vector of constants for the lin. eq. constrs.
         prodconstr(pr,v_all,v_all2, v_all3) coefficients for product constrs.
         startingpoint(v_all)    starting values
         feedflowdep(dv)         variables directly depending on feedflow;
$load linconstr, linconstrbound, eqconstr, eqconstrconst, prodconstr
$load startingpoint, feedflowdep
$Gdxin
*End of reading

*Declaring variables and equations
Variables
         x(v_all)        all variables
         z               objective;

Equations
         interpol(dv)    relations between independent and dependent variables
         linear(lc)      linear constraints
         eqlin(ec)       equality constraints
         product(pr, v_all, v_all2, v_all3)     products
         powerobjective  objective for maximizing power
         effobjective    objective for maximizing efficiency;
dv_all(v_all)=yes;       dv_all(iv)=no;
alias(iv, iv2);
* Setting the dependent variables with the regression equations
interpol(dv) ..  x(dv) =e= (.5*(high(dv)+low(dv))+(high(dv)-low(dv))*
         (betas("beta_0",dv) + sum(lb(iv,b), x(iv)*betas(b, dv)) +
         sum(qb(iv,b), x(iv)*x(iv)*betas(b, dv)) +
         sum(bb(iv,iv2,b), x(iv)*x(iv2)*betas(b,dv))))
         *(1 - feedflowdep(dv) + feedflowdep(dv) * sum(feedflow,x(feedflow)));

* Setting the linear constraints
linear(lc) ..    sum(iv$(linconstr(lc,iv)<>0),
         (x(iv)*(high(iv)-low(iv))+.5*(high(iv)+low(iv)))*linconstr(lc,iv)) +
         sum(dv_all$(linconstr(lc,dv_all)<>0), x(dv_all)*linconstr(lc, dv_all))
         =l= linconstrbound(lc);
eqlin(ec) ..    sum(iv$(eqconstr(ec,iv)<>0),
         (x(iv)*(high(iv)-low(iv))+.5*(high(iv)+low(iv)))* eqconstr(ec,iv)) +
         sum(dv_all$(eqconstr(ec,dv_all)<>0), x(dv_all)*eqconstr(ec, dv_all))=e=
         eqconstrconst(ec);
Parameter coeff(v_all), const(v_all);
coeff(v_all)=1;                          const(v_all)=0;
coeff(iv)=high(iv)-low(iv);              const(iv)=.5*(high(iv)+low(iv));
* Setting product equality constraints
product(pr,v_all,v_all2, v_all3)$(prodconstr(pr,v_all,v_all2, v_all3)<>0) ..
         coeff(v_all)*x(v_all)+const(v_all) =e=
         prodconstr(pr,v_all,v_all2, v_all3)*
         (coeff(v_all2)*x(v_all2)+const(v_all2))*
         (coeff(v_all3)*x(v_all3)+const(v_all3));

* Lower and upper bounds
x.lo(v_bounded)=lower(v_bounded);
x.up(v_bounded)=upper(v_bounded);
*Normalizing the bounds for the input variables
x.lo(iv) = (lower(iv) - .5*(high(iv)+low(iv))) / (high(iv)-low(iv));
x.up(iv) = (upper(iv) - .5*(high(iv)+low(iv))) / (high(iv)-low(iv));

* Starting values
x.l(feedflow)=lower(feedflow);
x.l(iv) = (startingpoint(iv)-.5*(high(iv)+low(iv)))/(high(iv)-low(iv));
x.l(dv) = (1 - feedflowdep(dv) + feedflowdep(dv) * x.l("feedflow"))
         *.5*(high(dv)+low(dv))+(high(dv)-low(dv))*(betas("beta_0",dv) +
         sum(lb(iv,b), x.l(iv)*betas(b, dv)) +
         sum(qb(iv,b), x.l(iv)*x.l(iv)*betas(b, dv)) +
         sum(bb(iv,iv2,b), x.l(iv)*x.l(iv2)*betas(b,dv)));

* Including scripts for "other modules"
$include combustion.gms
$include air.gms
$include turbo.gms
$include hrsgconnected.gms

* Setting the objective
*** Net power output
powerobjective$(objtype eq 1) ..z =e= sum(v_all, objcoeffs(v_all)*x(v_all))
         + Q_H*Q_Hcost + Q_C*Q_Ccost + sum(s, flow(s)*flowcost(s));
*** Net efficiency
effobjective$(objtype eq 2) .. z*x("feedflow") =e=
         sum(v_all, objcoeffs(v_all)*x(v_all)) +
         Q_H*Q_Hcost + Q_C*Q_Ccost + sum(s, flow(s)*flowcost(s));

$ontext
model ircc /
         interpol
         linear
         eqlin
         product
         powerobjective
         effobjective
* equations from combustion.gms
         combustion
         heatbalance
* equations from air.gms
         airbalance
         ATRpressure
         airtemperaturesum
         AF
*equations from turbo.gms
         turbine1
         turbine2
         turbinetemp
         comprexp
* equations from hrconnected.gms
         T_inconnection
         T_outconnection
         flowconnection
         flowconnection2
* equations from hr.gms
*$ontext
         Q_H_eqn
         Q_balance
         T_m1h
         T_m2h
         T_m3h
         T_m1c
         T_m2c
         T_m3c
         Tisoh
         Tisoc
         Qpeqnh
         Qpeqn2h
         Qpeqnc
*$offtext
* equations from hr2.gms
*$ontext
         E2
         E3
         E4
         E5
         E6
*         E8
         E9
*         E10
         E11
         E12
         E13
         E14
         E14b
*         E15
         E16
         E17
*         E19
         E26
         E27
         E28
           y1_fix
           y3_fix
           y1fixc
           y1fixh
           y3fixc
           y3fixh
*$offtext
         /;
$offtext

model ircc /all/;

* Treat fixed variables as constant
ircc.holdfixed=1;
* Max solution time
ircc.reslim=maxtime;
* Some options
ircc.optcr=1e-9;
option sys12 = 1;
option limrow = 50;
option limcol = 50;
option minlp=baron;

* Setting bounds for the ATR pressure (used for manual branching)
*x.lo("feedpressure")=-.307;
*x.up=.5;

*Solving the optimization model
Solve ircc using minlp minimizing z;

*Preparing the solution to be written to file
Parameter x_opt(v_all);
x_opt(v_all)=x.l(v_all);
x_opt(iv)=.5*(high(iv)+low(iv))+(high(iv)-low(iv))*x.l(iv);

*Preparing information about the solution
Parameters       z_opt
                 exectime
                 iterations
                 modelstatus;
z_opt=z.l;
exectime=-5.0;
exectime=ircc.Resusd;
iterations=-5.0;
iterations=ircc.Iterusd;
modelstatus=ircc.ModelStat;


*Writing output to file
Execute_unload "irccoptimum.gdx" x_opt, z_opt, exectime, iterations, modelstatus
*** Heat integration solution
Parameters Tin(streams), Tout(streams), mCp(streams);
Tin(s)=T_in.l(s);
Tout(s)=T_out.l(s);
mCp(s)=flow.l(s);
Execute_unload "hrsgopt.gdx" Tin, Tout, mCp, HRAT, Q_H, Q_C, z.l;
