%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% OPTIMIZATION OF IRCC
%
% Functions/scripts used:
%   "HOME MADE": 
%   - regcoeffs.m (using polyreg2.m)
%   - setmodelparameters.m
%   - solveircc.m
%   - combustion.m
%   - air.m
%   - turbo.m
%   - findpos.m 
%   GDXMRW package:
%   - writegdx.m (using wgdx.m)
%   - readgdx.m (using rgdx.m)
%   - sp2full.m (and full2sp.m?)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
clear all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Control parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tighterbounds=0;    % 0: no tightening, 1: tighten bounds, 2: tighten more
tempscale=800;      % Scaling of temperatures in turbines and compressors
flowscale=3600e3;   % Scaling of all mass and energy flows
rungams=false;      % Run GAMS from MATLAB or not
maxtime=30;         % Max. solution time in seconds
runsimulation=false;% Do the HYSYS simulations or use saved simulation file
runregression=false;% Do regression or use saved regression coefficients
if runsimulation
    runregression=true;
end

objtype=2;          % 1: Maximize power, 2: Maximize efficiency
aircompressorfix=0; % 0: Try both, 1: Only second, 2: Only extra
lowerTIT=1285;         % 0: TIT = 1350 C, else TIT = lowerTIT [C]
otherLP=0;          % 0: LP = 4 bar, 8: LP = 8 bar
otherHP=0;          % 0: HP = 120 bar, 130: HP = 130 bar
relax=1;            % 0: No relaxation, 1: Include fewer pinch candidates

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting some basic parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
irccdata

if objtype==2
    objscale=1/CH4_in/LHV_CH4*100; % Scaling of the objective function
else
    objscale=1;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calling simulation to do run LHS sampling and HYSYS process simulations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if runsimulation
    simulation
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calling regcoeffs to fit data to 2nd order polynomials and provide GAMS
% with the regression coefficients
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
regcoeffs

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Starting point
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
startingpoint=[
    feedtemp        500
    LTStemp         250
    feedpressure    30
    HTStemp         300
    S2C             1
    airtemp         500
    steamtemp       345
    feedflow        8.7482];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calling setmodelparameters to provide GAMS with some linear constraints, 
% linear equality constraints, product constraints and a starting point
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
setmodelparameters
               
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting parameters related to combustion
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
combustion

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting parameters related to air supply
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
air

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting parameters related to turbines and compressors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
turbo

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting the objective function coefficients (the objective funtction is
% linear in the form cost*variable)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
objective=[...
%    feedflow            8e7/flowscale; ...
    CCU_CO2             w_CO2compr*split.CO2(2); ...
    fuelcompr           1; ...
    W_GT                -1; ...
    W_2ndcompr          1; ...
    W_extcompr          1; ...
    W_HPturb            -(1-STcondensercooling); ...
    W_IPturb            -(1-STcondensercooling); ...
    comb_airflow        w_air; ...
    flow_ATRair_2nd     w_air];
objective(:,2)=objscale*objective(:,2);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Setting lower and upper bounds for the variables
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
bounds

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Including sets and parameters related to heat integration
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
hr

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Writing objective, bounds and the set of all parameters to GDX file
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
writegdx('masterset.gdx', ...
    'set',       'v_all',       v_all, ...
    'set',       'v_bounded',   v_all, ...
    'parameter', 'lower',       [v_all lowerbound(v_all')'], ...
    'parameter', 'upper',       [v_all upperbound(v_all')'], ...
    'parameter', 'objcoeffs',   objective, ...
    'parameter', 'objtype',     objtype, ...
    'parameter', 'maxtime',     maxtime, ...
    uels);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Solving the model with GAMS/BARON
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
gamsdir='C:\Programfiler\GAMS23.6\';     %The directory where GAMS is installed
filepath='';
gamsfile=['"' filepath 'ircc.gms"']; %The gams file for the optimization
gamsoption=' MINLP BARON';             %Using BARON as MINLP solver
if rungams
    dos([gamsdir 'GAMSKEEP ' gamsfile gamsoption]);
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Reading the solution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
getsolution

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calculating other variables (postprocessing) and displaying the solution
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
irccoutput