%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% irccoutput.m - processing and displaying results (and reading some heat 
%                integration results)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions and scripts used:
% GDXMRW:   readgdx.m       reading data from.gdx files
%           sp2full.m       changing matrix from 'sparse' to 'full' format
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Names of structs
%   in  = Reformer feed
%   CSg = Cooled Syngas
%   SCC = Syngas to CCU
%   Con = Condensate
%   GTf = Gas Turbine fuel
%   CC  = Carbon Capture stream (CO2 to compression)
%   Ws  = Waste stream
% Unit: kmol/h

CH4_IN=CH4_in*x_opt(feedflow);

%Fixed values
in.CH4=CH4_IN/M.CH4;
CSg.O2=0/M.O2;
SCC.O2=0/M.O2;

% Values from x_opt
in.H2O=x_opt(S2C)*in.CH4;
CSg.CO=x_opt(CS_CO)/M.CO;
CSg.CO2=x_opt(CS_CO2)/M.CO2;
CSg.H2=x_opt(CS_H2)/M.H2;
SCC.CO=x_opt(CCU_CO)/M.CO;
SCC.CO2=x_opt(CCU_CO2)/M.CO2;
SCC.H2=x_opt(CCU_H2)/M.H2;
SCC.CH4=x_opt(CCU_CH4)/M.CH4;
SCC.H2O=x_opt(CCU_H2O)/M.H2O;
SCC.N2=x_opt(CCU_N2)/M.N2;

% Atom balance over reformer and shift reactors
CSg.CH4=in.CH4-CSg.CO-CSg.CO2;                  % Carbon balance
CSg.H2O=in.H2O-CSg.H2+2*(in.CH4-CSg.CH4);       % Hydrogen balance
in.O2=CSg.O2+CSg.CO2+.5*(CSg.CO+CSg.H2O-in.H2O);% Oxygen balance
in.N2=.79/.21*in.O2;                            % Air composition
CSg.N2=in.N2;                                   % Nitrogen balance


% Calculating the streams after the CCU split, and the condensate
for j=components
    i=char(j);
    % Condensate from water removal/separator
    eval(['Con.' i '=CSg.' i '-SCC.' i ';']);
    % Streams out of CCU
    eval(['GTf.' i '=SCC.' i '*split.' i '(1);']);
    eval(['CC.' i '=SCC.' i '*split.' i '(2);']);
    eval(['Ws.' i '=SCC.' i '*split.' i '(3);']);
end

% Displaying the solution
frmt='%35s %10.4g %12.4g %8s \n';
obj=sp2full(objective,'parameter');
if length(obj)<V
    obj(V)=0;
end
disp('Variable                              Obj.coeff.  Optimal value')
for i=v_all'
    if upperbound(i)==lowerbound(i)
        te='Fixed';
    elseif x_opt(i)==upperbound(i)
        te='UPPER';
    elseif x_opt(i)==lowerbound(i)
        te='lower';
    else
        te=num2str((x_opt(i)-lowerbound(i))/(upperbound(i)-lowerbound(i)),'%6.4f');
    end
    fprintf(frmt, uels{i}, obj(i), x_opt(i),te);
end

% Displaying molar flow of components in GT fuel and carbon capture stream
GTf
CC
disp('[kmol/h]')

% Calculating and displaying excess air ratio
disp(['Excess air ratio:     ' ...
    num2str(x_opt(exh_O2)/(x_opt(comb_airflow)*airO2mass-x_opt(exh_O2)))...
    '   (' num2str(x_opt(exh_O2)/sum(x_opt(exhaust_set))*100) ...
    ' % O2 by mass in exhaust)']);

% Displaying results for turbines and compressors
disp('Turbines and compressors')
frmt='%16s %16s %16s %16s %16s \n';
fprintf(frmt,uels{trb_set})
frmt='%16.6g %16.6g %16.6g %16.6g %16.6g \n';
fprintf(frmt,x_opt(outlettemp_set)*tempscale-C2K)
fprintf(frmt,x_opt(p_out_set))
fprintf(frmt,x_opt(W_set))
fprintf(frmt,x_opt(kappafrac_set))
fprintf(frmt,sum(x_opt(exhaust_set)), x_opt(flow_ATRair_set), x_opt(HPsteamflow), x_opt(IPsteamflow))

% Reading some heat integration results
mCp_opt=sp2full(readgdx('hrsgopt.gdx','mCp'),'parameter')';
try
    Q_H_opt=readgdx('hrsgopt.gdx','Q_H')';
    if length(Q_H_opt)<1;Q_H_opt=0;end
catch
    Q_H_opt=0;
end
try
    Q_C_opt=readgdx('hrsgopt.gdx','Q_C')';
    if length(Q_C_opt)<1;Q_C_opt=0;end
catch
    Q_H_opt=0;
end

% Calculating power output and consumtions
W_air=w_air*airflow;
W_LP=w_LP*(mCp_opt(LPB)/q_LPB+x_opt(IPsteamflow));
W_CO2compr=w_CO2compr*split.CO2(2)*x_opt(CCU_CO2);
condenserpumpwork=STcondensercooling*(x_opt(W_HPturb)+x_opt(W_IPturb)+W_LP);
pumpwork=(w_HPpump+w_LPpump)*x_opt(HPsteamflow)+w_LPpump*mCp_opt(LPB)/q_LPB;
poweroutput = x_opt(W_GT)-W_air-x_opt(W_2ndcompr)-x_opt(W_extcompr)+ ...
  x_opt(W_HPturb) + x_opt(W_IPturb) + W_LP - x_opt(fuelcompr) - ...
  condenserpumpwork - pumpwork - W_CO2compr - ...
  Q_C_opt*Q_Ccost/objscale - Q_H_opt*Q_Hcost/objscale;
W_aux=condenserpumpwork + pumpwork +...
    Q_Ccost*Q_C_opt/objscale + Q_Hcost*Q_H_opt/objscale;
disp('POWER OUTPUT SUMMARY')
frmt='%36s %9.2f %4s \n';
unit='MW';
captureratio= x_opt(CCU_CO2)*split.CO2(2)/M.CO2*M.CH4./CH4_IN;
powertabledata=[x_opt(W_GT) -W_air -x_opt(W_2ndcompr) -x_opt(W_extcompr)...
    x_opt(W_HPturb) x_opt(W_IPturb) W_LP -W_CO2compr -x_opt(fuelcompr) ...
    -W_aux poweroutput LHV_CH4*CH4_IN poweroutput/(LHV_CH4*CH4_IN)*100 ...
    -Q_C_opt -Q_H_opt captureratio x_opt(feedpressure) x_opt(S2C)]';
fprintf(frmt, 'Gas turbine power (gross):', x_opt(W_GT), unit)
fprintf(frmt, 'Main air compressor work:', -W_air, unit)
fprintf(frmt, 'Second air compressor work:', -x_opt(W_2ndcompr), unit)
fprintf(frmt, 'Extra air compressor work:', -x_opt(W_extcompr), unit)
fprintf(frmt, 'HP steam turbine power:', x_opt(W_HPturb), unit)
fprintf(frmt, 'IP steam turbine power:', x_opt(W_IPturb), unit)
fprintf(frmt, 'LP steam turbine power:', W_LP, unit)
fprintf(frmt, 'CO2 compression:', -W_CO2compr, unit)
fprintf(frmt, 'GT fuel compression work:', -x_opt(fuelcompr), unit)
fprintf(frmt, 'Condenser cooling (work)', -condenserpumpwork, unit)
fprintf(frmt, 'Pumps for LP and HP water:', -pumpwork, unit)
fprintf(frmt, 'Cooling water pumps:', -Q_Ccost*Q_C_opt/objscale, unit)
fprintf(frmt, 'External heating:', -Q_Hcost*Q_H_opt/objscale, unit)
disp('------------------------------------------------------------------')
fprintf(frmt, 'Net power output:', poweroutput, unit)
disp('==================================================================')
fprintf(frmt, 'Energy input (CH4 LHV):', LHV_CH4*CH4_IN, unit)
disp('==================================================================')
fprintf(frmt, 'Efficiency:', poweroutput/(LHV_CH4*CH4_IN)*100, '%')
disp('==================================================================')
fprintf(frmt, 'Cold utilities (external cooling):', -Q_C_opt, unit)
fprintf(frmt, 'Hot utilities (external heating):', -Q_H_opt, unit)
disp('==================================================================')
fprintf(frmt, 'CO2 capture ratio:', captureratio*100, '%')
fprintf(frmt, 'ATR pressure:', x_opt(feedpressure), 'bar')
fprintf(frmt, 'Steam to carbon ratio:', x_opt(S2C), '')

disp(info(1));
modelstat={'Optimal', 'Locally Optimal', 'Unbounded', 'Infeasible', ...
        'Locally Infeasible', 'Intermediate Infeasible', ...
        'Intermediate Nonoptimal', 'Integer Solution', ...
        'Intermediate Non-Integer', 'Integer Infeasible', ...
        'Licencing Problems - No Solution', 'Error Unknown', ...
        'Error No Solution', 'No Solution Returned', 'Solved Unique', ...
        'Solved', 'Solved Singular', 'Unbounded - No Solution', ...
        'Infeasible - No Solution'};
disp(['Model status ' num2str(info(4)) ' - ' modelstat{info(4)}]);