function [betas RSS] = polyreg2(x,y, varargin)
%POLYREG2 2nd order polynomial regression
%   POLYREG2(x,y) performs a second order polynomial regression, trying to
%   fit the data as, y_hat=f(beta_hat,x)
%       x(s,v)      = value of input variable v in simulation s
%       y(s)        = value of the output variable from simulation s
%   POLYREG2(x,y,noreg) performs the regression "without" the variables
%       represented by noreg)
%   POLYREG2(x,y,noreg,limit) does the same as POLYREG2(x,y,noreg), but
%       eliminating betas with lower absolute value than specified by limit
    
    %Setting up the C matrix (with 2nd degree polynomials)
    [sims iv]=size(x);
    n=1+iv*(1.5+.5*iv); % number of betas (regression coefficients)
    C=zeros(sims,n);
    for s=1:sims
        C(s,1)=1;                               % Constant term
        for v=1:iv
            C(s,1+v)=x(s,v);                    % First-order terms
            C(s,iv+1+v)=x(s,v)^2;               % Second-order terms
        end
        jj=2+2*iv;
        for v1=1:iv-1;
            for v2=v1+1:iv
                C(s,jj)=x(s,v1)*x(s,v2);    % Bilinear terms
                jj=jj+1;
            end
        end
    end
    
    % The limit to be used if not specified by optional argument
    limit=5e-2;
    
    % Dealing with optional arguments
    if size(varargin,2)>=1
        % Removing regression coefficients as set by 'noreg'
        if sum(varargin{1})>0
            for v=1:iv
                check{v}=[v+1 v+iv+1];
            end
            jj=2+2*iv;
            for v1=1:iv-1;
                for v2=v1+1:iv
                    check{v1}=union(check{v1},jj);
                    check{v2}=union(check{v2},jj);
                    jj=jj+1;
                end
            end
            for i=find(varargin{1},iv)
                C(:,check{i})=0;
                for j=check{i}
                    sims=sims+1;
                    C(sims,j)=1;
                    y(sims)=0;
                end
            end
        end
        % Setting the limit to the corresponding optional argument
        if size(varargin,2)>=2
            limit=varargin{2};
        end
    end
    
    for loop=1:5
        % Solving with respect to least squares
        betas = C\y;
        %       betas is now the solution (with respect to least squares)
        %       of the equation set C*betas=y.
        RSS = (y-C*betas)'*(y-C*betas); %The residual sum of squares

        % Removing "insignificant" coefficients
        % 1.    Adjusting the coefficients to account for that the
        %       variables are in the range [-.5, .5]
        betas_adj=[betas(1)*4;betas(2:iv+1)*2;betas(iv+2:2*iv+1);...
            betas(2*iv+2:n)];
        % 2.    Determining which coefficients too keep
        keep=abs(betas_adj)>max(betas_adj)*limit; 
        if sum(betas.*(1-keep))==0
            return;
        end
        for i=1:n
            if keep(i)==0 & betas(i)~=0
                % Removing the impact of beta(i) from the regression
                % equations
                C(:,i)=0;
                
                % Making a new constraint to make sure beta(i) is
                % set to zero (when minimizing least squares)
                sims=sims+1;
                C(sims,:)=zeros(1,n);
                C(sims,i)=1;
                y(sims)=0;
            end
        end
    end
    
    % In case there are more "insignificant" regression coefficients left
    % after the loops: show it.
   for i=1:n
       if abs(betas(i))<1e-12
           betas(i)=0;
       end
   end
   if sum(betas~=betas.*keep);
    disp('WARNING: There are still insignificant regression coefficients.')
    disp(betas.*(1-keep))
   end