%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% simulation.m - running similations in HYSYS based on LHS sampling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions and scripts used:
% HOME MADE:    sampling.m      making Latin hypercube sampling scheme
%               hywait.m        waiting for HYSYS to be ready (necessary?)
%                               - using hyissolving.m from hysyslib
% HYSYSLIB:     hyconnect.m     connecting to HYSYS case
%               hyspread.m      connecting to spreadsheet in HYSYS case
%               hycell.m        connecting to cell in HYSYS spreadsheet
%               hyvalue.m       getting value of a HYSYS spreadcheet cell
%               hyunits.m       getting unit for a HYSYS spreadcheet cell
%               hyrelease.m     releasing the HYSYS case
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% If run separately, remember to run irccdata first

% Connecting to HYSYS case and spreadsheet
hysys=hyconnect([pwd '\ircc7.hsc']);
sheet=hyspread(hysys,'I/O');

hywait(hysys)

% Connecting to cells of the HYSYS spreadsheet
hc=cell(1,v);
for i=1:v
    hc{i}=hycell(sheet,['A' num2str(i)]);
end

% Defining feasible ranges for input variables
low=zeros(1,iv);
high=zeros(1,iv);
step=ones(1,iv);
low(feedtemp)=200;
high(feedtemp)=500;
low(LTStemp)=180;
high(LTStemp)=250;
low(feedpressure)=18;
high(feedpressure)=30;
low(HTStemp)=300;
high(HTStemp)=450;
low(S2C)=1;
high(S2C)=2;
low(airtemp)=410;
high(airtemp)=530;
low(steamtemp)=280;
high(steamtemp)=345;

% Changing units
unitmult=ones(1,iv);
if strcmp(hyunits(hc{feedpressure}{1}),'kPa')
    unitmult(feedpressure)=100;
end
unitmult(S2C)=10000*M.H2O/M.CH4/3600;
low=low.*unitmult;
high=high.*unitmult;

% Widening the ranges to avoid bad fit around the bounds
a=.2;
high=(1+a)*high-a*low;
low=(1+a)*low-a*high;

% Setting the number of simulations to be done
sims=5000;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Calling the function sampling.m to make a latin hypercube sampling scheme
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
sample=(sampling(sims,iv)-rand(sims,iv))./sims*diag(high-low) ...
    +ones(sims,1)*low;

% Running HYSYS for the different values of the input variables, and
% saving the results for all regression variables in the matrix A
A=zeros(sims,v);
ii=1;
sample(:,feedpressure)=sample(:,feedpressure)-(sample(:,feedpressure)>3000).*(sample(:,feedpressure)-3000);
for si=1:sims
    for j=1:iv
        hyset(hc{j},sample(si,j));
    end
    hywait(hysys)
    for jj=1:v
        A(ii,jj)=hyvalue(hc{jj});
    end
    ii=ii+1;
end
hyrelease(hysys);

% Modifying the A matrix
% Changing from kPa to bar
A(:,feedpressure)=A(:,feedpressure)*.01;
% Scaling mass and energy flows
A(:,regvarflows)=A(:,regvarflows)/flowscale;
% Going from steam flow rate to steam-to-carbon ratio
A(:,S2C)=A(:,S2C).*M.CH4./(M.H2O*CH4_in*flowscale);
% Going from cooling/heating duty to mCp-flow [HRSG]
A(:,mCp_refprod)=A(:,mCp_refprod)./(T_refprod - A(:,HTStemp))*3600;
A(:,mCp_HTSprod)=A(:,mCp_HTSprod)./(A(:,HTStemp2) - A(:,LTStemp))*3600;
A(:,mCp_LTSprod)=A(:,mCp_LTSprod)./(A(:,LTStemp2) + C2K - T_WRfeed)*3600;
A(:,mCp_GTfuel)=A(:,mCp_GTfuel)./(200-30)*3600;
% Changing from kW to kJ/h
A(:,fuelcompr)=A(:,fuelcompr)*3600;

% Saving the matrix A in A_matrix7.mat
save('A_matrix7', 'A')