%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% turbo.m - setting parameters and sets related to turbines and compressors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Functions and scripts used:
% GDXMRW:       writegdx.m      writing data to gdx file
% and indirectly the home made script HPturbCp.m for simulations and
% regression for HP and IP steam turbines
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Fixed temperatures
T_air= 415+C2K;  % [K]
TIT=  1350+C2K;  % [K]
T_fuel=200+C2K;  % [K]
T_air_2nd=T_air; % [K]
T_air_ext=15+C2K;% [K]
T_HP = 560 + C2K;% [K]
T_RH = 560 + C2K;% [K]
p_air_2nd=18;  %[bar]
p_air_ext=1.01; %[bar]
HP=120;         %[bar]
LP=4;           %[bar]

if lowerTIT>0
    TIT=lowerTIT+C2K;
end

Cpcoeffs=[  HCp_coeffs.H2O
            HCp_coeffs.CO2
            HCp_coeffs.O2
            HCp_coeffs.N2
            airO2mass*HCp_coeffs.O2+airN2mass*HCp_coeffs.N2
            airO2mass*HCp_coeffs.O2+airN2mass*HCp_coeffs.N2
            HCp_coeffs.H2O
            HCp_coeffs.H2O];

%TEMPSCALE
Cpcoeffs=Cpcoeffs*diag(tempscale.^(1:5));
TIT=TIT/tempscale;
T_air_2nd=T_air_2nd/tempscale;
T_air_ext=T_air_ext/tempscale;
T_HP=T_HP/tempscale;
T_RH=T_RH/tempscale;

% Some constants
Combustionpressure=18;              %[bar]
Outletpressure=1.03;                %[bar]
Pressureloss=.5;                    %[bar]
Turbineefficiency=.91;
Compressorefficiency=.88;
HPturbineefficiency=.94;
IPturbineefficiency=.92;
Mainaircompressorefficiency=.92;

% The work of CO2 compression, per kg of CO2 captured
w_CO2compr=95*3.6; % [kJ/kg]

% Calculating the work of compressing 1 kg of air to 18 bar
a=HCp_coeffs.O2;
T1=20+C2K; T2=415+C2K;
w_air=airO2mass*(a(1)*(T2-T1)+.5*a(2)*(T2^2-T1^2)+...
    a(3)/3*(T2^3-T1^3)+.25*a(4)*(T2^4-T1^4)+...
    .2*a(5)*(T2^5-T1^5));                               % [kJ/kg]
a=HCp_coeffs.N2;
w_air=w_air+airN2mass*(a(1)*(T2-T1)+.5*a(2)*(T2^2-T1^2)+...
    a(3)/3*(T2^3-T1^3)+.25*a(4)*(T2^4-T1^4)+...
    .2*a(5)*(T2^5-T1^5));                               % [kJ/kg]
w_air=w_air/Mainaircompressorefficiency;

Cpcoeffs_gdx=zeros(length(expset)*(length(trbflows_set)),3);
k=1;
for i=1:length(trbflows_set);
    for j=1:length(expset)
        Cpcoeffs_gdx(k,1)=trbflows_set(i);
        Cpcoeffs_gdx(k,2)=expset(j);
        Cpcoeffs_gdx(k,3)=Cpcoeffs(i,j);
        k=k+1;
    end
end

GT_set=length(uels)+1;
secondcompressor_set=GT_set+1;
extracompressor_set=secondcompressor_set+1;
HPsteamturb_set=extracompressor_set+1;
IPsteamturb_set=HPsteamturb_set+1;
uels{GT_set}='Gas turbine';
uels{secondcompressor_set}='Second compressor';
uels{extracompressor_set}='Extra compressor';
uels{HPsteamturb_set}='HP steam turbine';
uels{IPsteamturb_set}='IP steam turbine';

% The set of turbomachinery to be modelled with thermodynamics equations
trb1_set=[GT_set secondcompressor_set extracompressor_set]';
% The set of turbomachinery to be modelled with linear regression from
% simulations
trb2_set=[HPsteamturb_set IPsteamturb_set]';
% The set of turbines and compressors modelled in one of these ways
trb_set=[trb1_set; trb2_set];

% Connecting flowrates to turbines/compressors
turboflows=[GT_set*ones(4,1) exhaust_set ones(4,1);
    [secondcompressor_set;extracompressor_set] flow_ATRair_set ones(2,1);
    HPsteamturb_set     HPsteamflow     1;
    IPsteamturb_set     IPsteamflow     1];

% Efficiencies for the turbines/compressors
efficiency=[GT_set              Turbineefficiency;
            secondcompressor_set -1/Compressorefficiency;
            extracompressor_set -1/Compressorefficiency;
            HPsteamturb_set     HPturbineefficiency;
            IPsteamturb_set     IPturbineefficiency];
% Inlet pressures for the turbines/compressors
p_in=[  GT_set                  Combustionpressure-Pressureloss;
        secondcompressor_set    p_air_2nd;
        extracompressor_set     p_air_ext;
        HPsteamturb_set         HP;
        IPsteamturb_set         LP]; %This is really the outlet pressure
% Inlet temperatures for the turbines/compressors
inlettemps=[GT_set              TIT;
        secondcompressor_set    T_air_2nd;
        extracompressor_set     T_air_ext;
        HPsteamturb_set         T_HP;      
        IPsteamturb_set         T_RH];
% Parameter to modify turbo-equations
kappafracsign=[GT_set           1;
        secondcompressor_set    1;
        extracompressor_set     1;
        HPsteamturb_set         1;
        IPsteamturb_set         -1]; %This interchanges p_in and x(p_out)

% Universal gas constant [kJ/kmol/K]
R=8.314472;
% 'Molar mass of air' [kg/kmol]
M.air=airO2*M.O2+airN2*M.N2;
% Gas constants with adjusted for molar mass [kJ/kg/K]
R_m=[trbflows_set R./[M.H2O M.CO2 M.O2 M.N2 M.air M.air M.H2O M.H2O]'];


% Setting regression coefficients for the turbines/compressors modelled
% with linear regression and simulation. The numbers are calculated in
% HPturbCp.m
% The equations are on the form w(p) = w0 + dwdp*p and T(p) = T0 + dTdp*p.
w0=[    HPsteamturb_set        694.90347928384426;
        IPsteamturb_set         279.90610444742094];
dwdp=[  HPsteamturb_set         -9.937395471077858;
        IPsteamturb_set         9.7361026004373343];    
T0=[    HPsteamturb_set         (186.53771204510576+C2K)/tempscale;
        IPsteamturb_set         (432.07460037381867+C2K)/tempscale];
dTdp=[  HPsteamturb_set         5.3325971476587695/tempscale;
        IPsteamturb_set         -5.1884382644666687/tempscale];
% Changing these data if the a different high pressure level is chosen
if otherHP==130
    w0=[    HPsteamturb_set         704.5183;
            IPsteamturb_set         57.0137];
    dwdp=[  HPsteamturb_set         -9.6975;
            IPsteamturb_set         10.5734];
    T0=[    HPsteamturb_set         (178.3142+C2K)/tempscale;
            IPsteamturb_set         (478.5753+C2K)/tempscale];
    dTdp=[  HPsteamturb_set         5.2336/tempscale;
            IPsteamturb_set         -5.5605/tempscale];
end

% Coefficient to find condenser cooling work
STcondensercooling=.005; % 0.5 percent of power from steam turbines

% Writing data to file
writegdx('turbo.gdx', ...
    'set',       'v_all',        v_all, ...
    'set',       'trb',          trb_set, ...
    'set',       'trb1',         trb1_set, ...
    'set',       'trb2',         trb2_set, ...
    'set',       'T_trb_out',    outlettemp_set, ...
    'set',       'Ti_trb_out',   idealoutlettemp_set, ...
    'set',       'W',            W_set, ...
    'set',       'trbflows',     trbflows_set, ...
    'set',       'p_out',        p_out_set, ...
    'set',       'exp',          expset, ...
    'set',       'kappafrac',    kappafrac_set, ...
    'parameter', 'turboflows',   turboflows, ...
    'parameter', 'T_trb_in',     inlettemps, ...
    'parameter', 'p_in',         p_in, ...
    'parameter', 'exponents',    [expset (1:5)'], ...
    'parameter', 'Cpcoeffs',     Cpcoeffs_gdx, ...
    'parameter', 'R',            R_m, ...
    'parameter', 'eff',          efficiency, ...
    'parameter', 'kappafracsign',kappafracsign, ...
    'parameter', 'w0',           w0, ...
    'parameter', 'dwdp',         dwdp, ...
    'parameter', 'T0',           T0, ...
    'parameter', 'dTdp',         dTdp, ...
    uels);